(function() {

    /**
     * Classe para controlar a exibição do conteúdo de notícia
     *
     * @class Rss
     * @constructor
     * @param {String} projPath Caminho do projeto
     */
	function Rss(projPath, areaMain) {
		this.templateNameToVar = {
			'template' 	: 'template',
			'texto' 	: 'text',
			'url'		: 'url',
			'imagem' 	: 'image',
			'duration' 	: 'duration',
			'startDate' : 'starDate',
			'endDate'	: 'endDate'
		};
		this.templateTypes = ['html', 'swf'];
        this.projPath = projPath || '';
		this.areaMain = areaMain;
	}

    /**
     * Método para setar o path do projeto
     *
     * @method setProjPath
     * @param {string} path
     */
    Rss.prototype.setProjPath = function (path) {
        this.projPath = path;
    }

    /**
     * Método para setar o json com o conteúdo a ser tocado
     *
     * @method setItens
     * @param {Object} itens Json dos conteúdos a serem tocados
     */
	Rss.prototype.setItens = function(itens) {
		this.itens = itens;
	}

    /**
     * Método para retornar qual item será tocado
     *
     * @method next
     * @return {integer} Índice do item a ser tocado
     */
	Rss.prototype.next = function() {
		// console.log('Rss Length:' + this.itens.length);

		if(typeof(this.current) == 'undefined')
			return this.current = 0;

		if(this.current >= this.itens.length -1)
			return this.current = 0;

		this.current = this.current + 1;
		return this.current;
	}

    /**
     * Método para retornar qual elemento será tocado
     *
     * @method getItem
     * @param  {Integer} current Índice do item a ser tocado
     * @return {Object}         Elemento json a ser tocado
     */
	Rss.prototype.getItem = function(current) {
		var item;
		if(!current)
			var current = this.current;
		item = this.itens[current];

		//console.log('Rss getItem' + item.texto);
		return item;
	}

    /**
     * Método para retornar qual será o próximo item a ser tocado
     *
     * @method getNext
     * @param  {Function} callBack Função de callback
     * @param  {Integer} loop     Id do item a ser tocado
     * @return {void}
     */
	Rss.prototype.getNext = function(callBack, loop) {
		// console.log('getNext loop', loop);

        // verifica se existe lista
        // TODO: caso caia aqui nesse item, a notícia deve ser reiniciada
        // o problema é que o arquivo ainda não existe quando esse template é chamado
        if (_.isUndefined(this.itens)) {
            // para evitar um loop sem controle, tem um delay de 4s
            return setTimeout(function() {
                callBack('No Rss feed not found');
            }, 4000);
        }

		if(loop == undefined)
			var loop = this.itens.length;
		else if(loop == 0)
			return callBack('No Rss feed valid');

		if((typeof(this.itens) != 'object') || !this.itens.length)
			return callBack('Rss feed list is empty');

		if(this.itens.length == 0)
			return callBack('Rss feed list is empty');

		//Restart the rss feed pointer
		if(typeof(this.current) == 'undefined' || this.current >= this.itens.length - 1)
			this.current = 0;
		else
			this.current++;

		var that = this;
        var item = this.itens[this.current];

        this.validateRss(item, function(isAvailable){
			if(isAvailable)
				return callBack(item);
			else
				that.getNext(callBack, loop-1);
		});
	}

    /**
     * Método para verificar se o coteúdo a ser tocado é válido
     *
     * @method validateRss
     * @param  {Object} item     Json do item a ser tocado
     * @param  {Function} callBack Função de callback
     * @return {void}
     */
	Rss.prototype.validateRss = function(item, callBack){
		this.validateDate(item, callBack);
	}

	/**
	 * Method to check new validate date
     *
     * @method validateDate
	 * @param Json item
	 * @return Boolean
	 */
	Rss.prototype.validateDate = function(item, callBack){
		if(!item)
			return callBack(false);

		var now = new Date().getTime();
		var startDate = new Date(item.startDate).getTime();
		var endDate = new Date(item.endDate).getTime();

		//Let the pass, if startDate and endDate is not a number
		if(isNaN(startDate) && isNaN(endDate))
			return this.isTemplateAvailable(item, callBack);

		//Make sure the item is between time interval
		if((now >= startDate || isNaN(startDate) ) && (now <= endDate || isNaN(endDate) ))
			return this.isTemplateAvailable(item, callBack);

		return callBack(false);
	}

    /**
     * Método para verificar se a notícia a ser tocada tem imagem
     * @param  {json}  item
     * @param  {Function}  callBack
     * @return {void}
     */
	Rss.prototype.isImgNewAvailable = function(item, callBack){
        // tipo padrão para layout
        var typeDownload = FileSync.FILE_TYPE_LAYOUT_RSS_IMG;
        var imgPath = FileSync.PLAYER_DIR_LAYOUT_RSS_IMG;

        // caso seja área principal o caminho para download da imagem é diferente
        if (this.areaMain) {
            typeDownload = FileSync.FILE_TYPE_RSS_IMG;
            imgPath = FileSync.PLAYER_DIR_RSS_IMG;
        }

        imgPath = this.projPath + '/' + imgPath + '/' + item.imagem;
		// console.log('isImgNewAvailable', item.imagem);

		if(item.imagem == '0.png')
			return callBack(true);

		FileSync.exists(imgPath, function(imgFileEntry, error){

			if(error || !imgFileEntry){
                // download
				var downloadItem = {file: item.imagem, type: typeDownload, url: null, notify: false, trying: 0};
				DownloadCenter.addDownload(downloadItem);
			}

			return callBack(true);
		});
	}

	Rss.prototype.isTemplateAvailable = function(item, callBack){
		var templateExt = item.template.split('.').pop();
		var templateFolder = item.template.split('.')[0];
		var that = this;

        // padrão para layout
        var typeDownload = FileSync.FILE_TYPE_LAYOUT_TEMPLATE;
        var folderTemplate = FileSync.PLAYER_DIR_LAYOUT_TEMPLATE;


        // não toca coneúdo do flash
        if(templateExt == 'swf')
            return callBack(false);

        // caso seja área principal o caminho para download é diferente
        if (this.areaMain) {
            typeDownload = FileSync.FILE_TYPE_RSS_TEMPLATE;
            folderTemplate = FileSync.PLAYER_DIR_RSS_TEMPLATE;
        }

        var templatePath = this.projPath + '/' + folderTemplate + '/' + templateFolder + '/index.html';
		FileSync.exists(templatePath, function(templateFileEntry, error){

			if(error || !templateFileEntry){
				var downloadItem = {file: item.template, type: typeDownload, url: null, notify: false, trying: 0};
				DownloadCenter.addDownload(downloadItem);

				return callBack(false);
			}
			else
				return that.isImgNewAvailable(item, callBack);
		});
	}

	Rss.prototype.getUrl = function(item) {
        // caso seja área auxiliar
        var imgPath = FileSync.PLAYER_DIR_LAYOUT_RSS_IMG;
        var folderTemplate = FileSync.PLAYER_DIR_LAYOUT_TEMPLATE;

        // caso seja área principal
        if (this.areaMain) {
            folderTemplate = FileSync.PLAYER_DIR_RSS_TEMPLATE;
            imgPath = FileSync.PLAYER_DIR_RSS_IMG;
        }

        // url
		var url = window.hostPlayerHttp +  this.projPath + '/' + folderTemplate + '/';

        //     // verifica se no objeto existe o template
        if (_.isObject(item) && ('template' in item)) {
            // seta o template
            this.template = item.template;
        }

        // caso não exista nenhum noticia
        if(_.isString(item)) {
            var templateExt = this.template.split('.')[0];

            return url + templateExt + '/index.html?imgVar=&txtVar=Nenhuma notícia encontrada';
        }

        var templateExt = item.template.split('.').pop();
        var templateFolder = item.template.split('.')[0];
        url = url + templateFolder + '/index.html?';

		// is HTML
		if(templateExt == 'zip') {
			var imgVar = 'imgVar='+ window.hostPlayerHttp + this.projPath + '/' + imgPath + '/' + item.imagem;
            // var texto = encodeURI(item.texto);
            var texto = jQuery("<div/>").html(item.texto).text();
			var txtVar = '&txtVar=' + texto;
			var url = url + imgVar + txtVar;

			_log('Notícia:', texto);
			return url;
		}

		// is SWF
		if(templateExt == 'swf') {
			return '';
		}

		return url + 'imgVar=&txtVar=Nenhuma notícia encontrada';
	}

    /**
     * Método para setar o template
     *
     * @method setTemplateLayout
     * @param {integer} template Arquivo do template
     */
    Rss.prototype.setTemplateLayout = function (template) {
        this.template = template;
    }

	/**
	 * Generete 4YouSee template URL
     *
     * @method toUrl
	 * @param  {[type]} name      [description]
	 * @param  {[type]} separator [description]
	 * @return {[type]}           [description]
	 */
	 Rss.prototype.toUrl = function (item, separator) {
		return this.serialize(item, separator);
	 }


	/**
	 * Serialize Url
     *
     * @method serialize
	 * @param  {[type]} name      [description]
	 * @param  {[type]} separator [description]
	 * @return {[type]}           [description]
	 */
	Rss.prototype.serialize = function (obj, separator) {
		var pairs = [];

		if(!separator)
			separator = '&';

		for (var prop in obj) {
			if (!obj.hasOwnProperty(prop)) {
				continue;
			}
			pairs.push(prop + '=' + obj[prop]);
		}
	 	return pairs.join(separator);
	}

	window.Rss = Rss;
})();
