/* globals _log, EventPlayer, FileSync, _logError, DownloadCenter, Player, jQuery, generateScriptText, scriptWebView */
'use strict';

(function (window) {

    /**
     * Classe para controlar a exibição dos conteúdos html
     *
     * @class HtmlPlayer
     * @constructor
     * @param {AppWindow} appWindow Objeto appWindow do chrome
     * @param {JSON} mediaFile Json do conteúdo
     * @param {Element} container Elemento html
     * @param {integer} time      Tempo de espera do conteúdo
     */
    function HtmlPlayer (appWindow, mediaFile, container, time)
    {
        this.appWindow = appWindow;
        this.mediaFile = mediaFile;
        this.container = container;
        this.time = time;
        this.appWindow.HtmlTimeOut = undefined;

        // para de tocar o conteúdo
        this.stopContent = false;
    }

    /**
     * Get method callback
     * @return {Function}
     */
    HtmlPlayer.prototype.getCallback = function __doneSetCallback()
    {
        var call = this.funcCallback;
        this.funcCallback = function() {
            _log("Callback já foi chamado.");
        };

        return call;
    };

    /**
     * Set method callback
     * @param {Function} functionCallback
     */
    HtmlPlayer.prototype.setCallback = function __doneSetCallback(functionCallback)
    {
        /**
         * Método de callback
         */
        this.funcCallback = function(error, nameFile) {
            playlistVideo.authChangePlayList = true;
            // callback
            functionCallback(error, nameFile);
        };
    };

    /**
     * Método para o player do conteúdo
     *
     * @method play
     * @param  {string} path        Caminho padrão
     * @param  {string} localPath   Caminho do arquivo
     * @return {void}
     */
    HtmlPlayer.prototype.play = function (path, localPath)
    {
        var that = this;
        var htmlUrl = getHtmlPath(that.mediaFile, path);
        var mediaPath = getHtmlPath(that.mediaFile, localPath);

        // console.log("HtmlPlayer play: mediaPath", mediaPath);

        // lê o arquivo
        FileSync.exists(mediaPath, function(fileExist){

            // em caso de erros
            if(!fileExist){
                // baixa os arquivos novamente
                var downloadItem = {file: that.mediaFile, type: FileSync.FILE_TYPE_MEDIA, url: null, notify: false, trying: 0};
                DownloadCenter.addDownload(downloadItem);

                // console
                var errorMsg = "Can't play html";
                _logError('--> HTML:', errorMsg, that.mediaFile);

                // reseta o timeout
                that.clearTimeout();

                // callback
                setTimeout(function() {
                    that.getCallback()(errorMsg, that.mediaFile);
                }, 5000);

                return;
            }

            // monta o conteúdo html na tela
            that.generateHtmlTag(htmlUrl, 'i', that.mediaFile);
        });

        /**
         * Método para retornar o item desejado
         *
         * @method getHtmlPath
         * @param  {string} mediaFile Nome do arquivo
         * @param  {string} path      Caminho
         * @return {string}
         */
        function getHtmlPath(mediaFile, path) {
            var htmlUrl;
            var mediaFolder = mediaFile.split('.')[0];
            var fileExt = mediaFile.split('.')[1];


            if(fileExt == 'zip') {
                htmlUrl = path + mediaFolder + '/index.html';
            }

            if(fileExt == 'html'){
                htmlUrl = path + mediaFile;
            }

            // console.log('getHtmlpath', htmlUrl);

            return htmlUrl;
        }
    };

    /**
         * Método para executar o html
         *
         * @method generateHtmlTag
         * @param  {string} htmlUrl     Url
         * @return {void}
         */
    HtmlPlayer.prototype.generateHtmlTag = function __generateHtmlTag (htmlUrl, type, mediaFile)
    {
        var that = this;
        var tagIframeLoadSuccess = false,
        setTimeoutTagIframeLoadSuccess;

        // element
        var webviewElement = this.container.querySelector('iframe');

        // remove todos os eventos
        removeEvent();

        // configura os parâmetro da webview
        Player.prototype.resizeContainer(webviewElement, this.container);

        // adiciona um evento
        webviewElement.addEventListener('load', eventLoadIframe, false);
        webviewElement.addEventListener('error', eventErrorIframe, false);
        jQuery(webviewElement).on('resetIframe', eventUnloadIframe);

        webviewElement.setAttribute('src', htmlUrl);

        // verifica se o iframe realmente está funcionando
        setTimeoutTagIframeLoadSuccess = setTimeout(function __done() {
            if (!tagIframeLoadSuccess) {
                // caso o iframe ainda não esteje pronto
                _logError('Aconteceu algum problema que o evento load do Iframe não foi disparado.');
                // remove todos os eventos
                removeEvent();
                // chama o próximo conteúdo
                that.getCallback()("Não foi possível tocar o conteúdo", mediaFile);
            }
        }, 8000);

        /**
         * Método para executar assim que for carregado
         * @return {void}
         */
        function eventLoadIframe() {
            // gamb, até descobrir porque o iframe chama o evento onload 2 vezes
            if (tagIframeLoadSuccess) {
                return;
            }

            Player.prototype.showContainer(that.container, 'iframe');
            Player.prototype.resizeContainer(webviewElement, that.container);

            // seta tag
            tagIframeLoadSuccess = true;
            clearTimeout(setTimeoutTagIframeLoadSuccess);

            // adicionando script na webview
            try {
                var script = document.createElement('script');
                script.text = generateScriptText(scriptWebView);
                webviewElement.contentDocument.body.appendChild(script);

                // verifica quando será necessário a troca de notícia
                postMessageIframe (htmlUrl, webviewElement, that.appWindow.id, type, mediaFile);
            } catch (exception) {
                _logError({
                    'Error': 'Erro ao tentar injetar scripts de controle do player no conteúdo html',
                    'htmlUrl': htmlUrl,
                    'file': exception.sourceURL,
                    'line': exception.line,
                });

                // remove todos os eventos
                removeEvent();
                // chama o próximo conteúdo
                return that.getCallback()("Não foi possível tocar o conteúdo", mediaFile);
            }

            // faz com que se necessário, todas as playlist de vídeo sejam geradas novamente
            playlistVideo.verifyCreatPlaylist();

            // console.log("HtmlPlayer: Troquei de conteudo.");
        }

        /**
         * Método que atende ao evento error do iframe
         * @return {void}
         */
        function eventErrorIframe() {

            _logError({
                'Type': 'HTML',
                'Event': 'error'
            });

            // reseta o timeout
            that.clearTimeout();

            // remove o evento
            removeEvent();

            // callback
            return that.getCallback()('Não foi possível carregar o html.', mediaFile);
        }

        /**
         * Evento que remove todos os eventos registrados para o iframe
         * @return {void}
         */
        function eventUnloadIframe() {
            // _log('---> HTML: Evento Unload sendo executado.');

            // remove a url
            webviewElement.setAttribute('src', '');

            // reseta o timeout
            that.clearTimeout();

            // remove o evento
            removeEvent();
        }

        /**
         * Método para remover os eventos do iframe
         * @return {[type]} [description]
         */
        function removeEvent() {
            // _log("---> HTML: Todos os eventos do sendo removidos.");

            // remove o evento
            webviewElement.removeEventListener('load', eventLoadIframe, false);
            webviewElement.removeEventListener('error', eventErrorIframe, false);
            jQuery(webviewElement).off('resetIframe', eventUnloadIframe);
        }


        // reseta o timeout
        this.clearTimeout();

        // time to out
        if(!this.time || this.time === 0)
            this.time = 15000;
        else
            this.time = this.time*1000;

        // permite tocar o conteúdo
        this.stopContent = false;

        /**
         * Encaminha mensagem para iframe, para que haja a troca de notícia
         * @param  {string} html
         * @param  {ObjectHtml} webview
         * @param  {ObjectHtml} idWindow
         * @param  {string} type Tipo da notícia | n -> notícia, i -> contéudo institucional
         * @return {void}
         */
        function postMessageIframe (html, webview, idWindow, type, mediaFile) {
            // verifica se é pra trocar a notícia
            that.appWindow.HtmlTimeOut = setTimeout(function sendMessageWebviewNews() {
                var post = {
                    typePlayer: window.parent.typePlayer,
                    main: true,
                    direction: 1,
                    container: that.container.id,
                    file: mediaFile,
                    type: type, // 'n' || 'i', // tipo de notícia é n
                    idWindow: idWindow,
                    keyArea: -1,
                    time: that.time,
                    news: html
                };

                // envia a nova notícia
                if (webview.style.opacity === "1" || that.appWindow.HtmlTimeOut !== undefined) {
                // if (!_.isNull(webview.contentWindow) || that.appWindow.HtmlTimeOut !== undefined) {
                    // O monitoramento da volta está no arquivo background.js:300
                   webview.contentWindow.postMessage(post, "*");

                   // chama a função denovo
                   postMessageIframe (html, webview, idWindow, type, mediaFile);
                }

            }, 2000);
        }
    };

    /**
     * Método para limpar o timeout
     *
     * @method clearTimeout
     * @return {void}
     */
    HtmlPlayer.prototype.clearTimeout = function ()
    {
        clearTimeout(this.appWindow.HtmlTimeOut);
        clearTimeout(this.timeOut);

        // para o conteúdo
        this.stopContent = true;
    };

    window.HtmlPlayer = HtmlPlayer;
}) (window);
