// scripts/Player/HtmlPlayerArea.js
(function (window) {
    /**
     * Classe para controlar a exibição do conteúdo html nas áreas do layout
     *
     * @class HtmlPlayerArea
     * @constructor
     * @param {Element} appWindow
     * @param {classRss} classRss
     * @param {Element} container Div da área
     * @param {float} time      Timeout
     */
    var HtmlPlayerArea = RssPlayer().clone();

    /**
     * Método para dar player no conteudo
     *
     * @method play
     * @param {function} callback Método para retorno no final da execução do conteúdo
     * @param {integer} keyArea Id da área que está executando o conteúdo
     * @return {void}
     */
    HtmlPlayerArea.prototype.play = function (callback, keyArea) {
        var that = this;

        /**
         * Variável para controlar o timeout que recarrega a área
         * @type {Object}
         */
        this.listAreaLoading = {};

        var endCallback = (typeof callback == "function"? callback : function HtmlPlayerAreaCallbackDone() {})

        /**
         * Exibe o proximo item
         * @param  {json} item
         * @return {void}
         */
        this.classRss.getNext(execGetNext, null);

        /**
         * Método para tratar o proxímo item a ser tocado
         *
         * @method execGetNext
         * @param  {Object} item
         * @return {void}
         */
        function execGetNext (item){

            // caso a notícia não seja válida
            if (!dateValidateRss(item)) { // função está no arquivo player.js
                // return that.classRss.getNext(execGetNext, null);
            }

            // em caso de erro
            if(typeof item === 'string' ){
                _logError({
                    "error": item,
                    "keyArea": keyArea,
                });
            }

            // verifica url
            var htmlUrl = that.classRss.getUrl(item);
            // em caso de error
            if(!htmlUrl) {
                _logError({
                    "message": "Can't play SWF rss",
                    "item": item,
                    "keyArea": keyArea,
                    "htmlUrl": htmlUrl,
                });
            }

            // caso o template ainda não tenha sido carregado
            if (_.isNull(that.containerWebview.getAttribute('src'))) {
                loadContent(that, htmlUrl, that.container, that.containerWebview);
            }

            // exibe conteúdo
            Player.prototype.showContainer(that.container, 'iframe', true);

            // chama a proxíma notícia
            that.changeNews(keyArea);

        }

        /**
         * Método para verificar se o conteúdo a ser tocado é válido
         *
         * @method dateValidateRss
         * @param  {Object} item
         * @return {boolean}
         */
        function dateValidateRss (item) {
            var today = new Date();
            var regex = new RegExp('-', 'g');

            var startDate = (!item.startDate) ? undefined : new Date(item.startDate.replace(regex, '/'));

            var endDate = undefined;
            if(item.endDate){
                endDate = new Date(item.endDate.replace(regex, '/'));
                endDate.setHours(23);
                endDate.setMinutes(59);
                endDate.setSeconds(59);
            }

            if(startDate != undefined && today < startDate)
                return false;

            if(endDate != undefined && today > endDate)
                return false;

            return true;
        };
    };

    /**
     * Método para trocar a notícias da área do layout
     * @param  {integer} keyArea
     */
    HtmlPlayerArea.prototype.changeNews = function (keyArea) {

        var that = this;

        this.classRss.getNext(function(item) {
            var htmlUrl = that.classRss.getUrl(item);

            // factory
            (function (html, webview, keyArea, idWindow, timeout) {

                // console.log("sendMessageWebviewNews: perguntando para " + webview.getAttribute("partition") + " se precisa trocar a notícia.");
                function sendMessageWebviewNews() {
                    var post = {
                        main: false,
                        idWindow: idWindow,
                        keyArea: keyArea,
                        time: that.time,
                        news: html
                    }
                    // envia a nova notícia
                    if (_.isNull(webview.contentWindow)) {
                        return;
                    }

                    // O monitoramento da volta está no arquivo background.js:300
                    webview.contentWindow.postMessage(post, "*");

                    // verifica se é pra trocar a notícia
                    timeout[keyArea] = setTimeout(sendMessageWebviewNews, 2000);
                }

                // chama a função
                timeout[keyArea] = setTimeout(sendMessageWebviewNews, 2000);

            }) (htmlUrl, that.containerWebview, parseInt(keyArea), that.appWindow.id, that.Timeout);

        }, null);
    };


    /**
     * Método para setar o novo container
     *
     * @method setConteiner
     * @param {Object} container Container
     */
    HtmlPlayerArea.prototype.setContainer = function (container) {
        if (!_.isObject(container)) {
            return;
        }

        this.container = container;
        this.containerWebview = container.querySelector('iframe');
    };

    /**
     * Método para resetar o container
     * @param  {integer} keyArea Chave do array onde está a classe
     * @return {void}
     */
    HtmlPlayerArea.prototype.resetContainer = function (keyArea) {
        // limpa url
        this.containerWebview.setAttribute('src', '');

        // inicia o novo conteúdo
        // Esse método está no arquivo scripts/backgroun.js
        playerContentRssLayout(keyArea, this.appWindow);
    };


    /**
     * Método para setar o template da área
     *
     * @method loadContent
     * @param  {string} htmlUrl Url
     * @param {Element} container Elemento html da área do conteúdo
     * @param {Element} containerWebview Elemento html da iframe do layout
     * @return {void}
     */
    function loadContent (that, htmlUrl, container, containerWebview) {
        var fileName = container.getAttribute('data-template');
        var codeTemplate = fileName.split('.')[0];

        // verifica se existe o layout a ser carregado
        var localPath = that.classRss.projPath + '/' + FileSync.PLAYER_DIR_RSS_TEMPLATE + '/' + codeTemplate + '/index.html';

        // busca a imagem
        FileSync.exists(localPath, function(foundFile) {
            htmlUrl +=  '&' + new Date().getTime();
            var errorFile = false;

            // em caso de erros
            if(!foundFile){
                // baixa os arquivos novamente
                var downloadItem = {file: fileName, type: FileSync.FILE_TYPE_LAYOUT_TEMPLATE, url: null, notify: false, trying: 0};
                DownloadCenter.addDownload(downloadItem);

                // recarrega a área
                resetAreaLayout (that, htmlUrl, container, containerWebview);

                // em caso de erros
                errorFile = true;
                htmlUrl = '';
            }

            // carrega layout
            loadAreaLayout (htmlUrl, container, containerWebview, errorFile);
        });

        /**
         * Método para resetar a área do layout
         *
         * @method resetAreaLayout
         * @param  {Object} that
         * @param  {string} htmlUrl
         * @param  {ObjectHtml} container
         * @param  {ObjectHtml} containerWebview
         * @return {void}
         */
        function resetAreaLayout (that, htmlUrl, container, containerWebview) {
            var timeout = 10;

            // caso não exista mais o iframe
            if (_.isNull(containerWebview)) {
                return;
            }

            // console
            var errorMsg = "Não foi possível carregar o template da área " + container.id + ". Será reiniciado em " + timeout + " segundos";
            _logError(errorMsg, fileName, localPath, {"listAreaLoading": that.listAreaLoading});

            // irá tentar recarregar o layout
            setTimeout(function __reloadAreaLayout() {
                // recarrega o conteúdo da área do layout
                loadContent(that, htmlUrl, container, containerWebview);
            }, timeout * 1000);
        }

        /**
         * Método para carregar o html do layout
         * @param  {string} htmlUrl Url
         * @param {Element} container Elemento html da área do conteúdo
         * @param {Element} containerWebview Elemento html da webview do layout
         * @return {void}
         */
        function loadAreaLayout (htmlUrl, container, containerWebview, errorFile) {
            // carrega o layout
            containerWebview.setAttribute('src', htmlUrl);
            containerWebview.addEventListener('load', load, false);
            containerWebview.addEventListener('error', eventErrorIframe, false);
            containerWebview.addEventListener ('unload', eventUnloadIframe , false);

            // exibe o conteudo
            Player.prototype.resizeContainer(containerWebview, container);
            Player.prototype.showContainer(containerWebview, 'iframe', true);

            /**
             * Método para remover os eventos do iframe
             * @return {[type]} [description]
             */
            function removeEvent() {
                // remove o evento
                containerWebview.removeEventListener('load', load, false);
                containerWebview.removeEventListener('error', eventErrorIframe, false);
                containerWebview.removeEventListener ('unload', eventUnloadIframe , false);
            }

            /**
             * Evento que remove todos os eventos registrados para o iframe
             * @return {void}
             */
            function eventUnloadIframe() {
                _log('Removendo conteúdo html');

                // reseta o timeout
                that.clearTimeout();

                // remove o evento
                removeEvent();
            }

            /**
             * Evento executado assim que iframe está carregado
             * @param  {Object} event
             * @return {void}
             */
            function load (event) {
                // 13-01-2015 TODO: deve ser removido.
                Player.prototype.resizeContainer(containerWebview, container);
                Player.prototype.showContainer(container, 'iframe', true);

                // injeta script no iframe
                try {
                    var script = document.createElement('script');
                    script.text = generateScriptText(scriptWebView);
                    containerWebview.contentDocument.body.appendChild(script);
                } catch (exception) {
                    _logError('Aconteceu um problema ao carregar a área do layout. Template:' + codeTemplate);

                    // remove todos os eventos
                    removeEvent();

                    // recarrega o layout
                    resetAreaLayout (that, htmlUrl, container, containerWebview);
                }


                // em caso de erros
                if (errorFile) {
                    // seta o layout
                    var div = document.createElement("div");
                    div.style.color = '#fff';
                    div.innerHTML = 'Wait, loading!';
                    containerWebview.contentDocument.body.appendChild(div);
                }
            }

            /**
             * Método que atende ao evento error do iframe
             * @return {void}
             */
            function eventErrorIframe() {

                _log({
                    'Type': 'HTML',
                    'Event': 'error'
                });

                // reseta o timeout
                that.clearTimeout();

                // remove os eventos
                removeEvent();

                // callback
                return endCallback('Não foi possível carregar o html.', htmlUrl);
            }
        }
    };


    window.HtmlPlayerArea = HtmlPlayerArea;
}) (window);
