/* globals _log, _logError, player4uc */
(function(_global) {
    'use strict';

    /**
     * Class for control brightness screen player
     * @todo Multi players
     *
     * @constructor ControllBrightness
     * @param {float} brightness
     */
    function ControllBrightness(brightness)
    {
        this.keyConfiguration = 'settings.brightness';
        this.brightness = brightness;
        this.setIntervalConfig = [];
        this.dataPlayer = player4uc.window.current().contentWindow.playerData;

        // set brightness
        this.set(brightness);

        // init configuration
        this.init();
    }

    /**
     * Init configurations for brightness
     * @method init
     */
    ControllBrightness.prototype.init = function __doneInit ()
    {
        var that = this;
        // clear timeouts
        this.clearTimeouts();

        // get config
        this.getConfigurationStorage(function __doneGet (config) {
            var now = new Date();

            // case object empty
            if (!(that.keyConfiguration in config) || _.isNull(config[that.keyConfiguration])) {
                _log("Settings empty.", config);
                config[that.keyConfiguration] = {};
                config[that.keyConfiguration][that.dataPlayer.id] = {itens: []};
            }

            config = config[that.keyConfiguration][that.dataPlayer.id].itens;

            //Caso não exista nenhuma faixa de brilho difine o valor padrão 85
            if(_.isEmpty(config)){
                that.set(85);
                return;
            }

            //Ordenando o objeto de acordo com a hora
            config = _.sortBy(config, 'init');

            for (var i in config) {
                // init
                var MinSec = config[i].init.split(":");
                var init = new Date();
                init.setHours(MinSec[0]);
                init.setMinutes(MinSec[1]);

                // caso date init be more now
                if (init < now) {
                    // set configuration
                    that.set(config[i].brightness);

                    // next
                    continue;
                }

                // return diff in minutes
                var timeOut = 1000 * Date.dateDiff('s', now, init);
                // register timeout
                that.registerBrightness(i, config[i].brightness, timeOut);
            }
        });
    };

    /**
     * Register brightness
     * @method registerBrightness
     *
     * @param {int} index
     * @param {int} brightness
     * @param {long} timeout
     */
    ControllBrightness.prototype.registerBrightness = function(index, brightness, timeout){
        var that = this;
        // register timeout
        console.log('Brightness Intensity', brightness);

        that.setIntervalConfig[index] = setTimeout(function __doneTimeOut() {
            // aplica as configurações
            that.set(brightness);

            // executa função para verificar se existe mais eventos a serem registrados
            that.init();
        }, timeout);
    };

    /**
     * Set brightness of screen player
     * @method set
     *
     * @param {float} brightness
     */
    ControllBrightness.prototype.set = function __doneSet (brightness)
    {
        var options = {};
        // Expert1 mode.
        options.backlight = brightness;
        // options.contrast = 50;
        options.brightness = brightness;
        // options.hSharpness = 50;
        // options.vSharpness = 50;
        // options.color = 50;
        // options.tint = 50;
        // options.dynamicContrast = "low";
        // options.superResolution = "low";
        // options.colorGamut = "wide";
        // options.dynamicColor = "high";
        // options.noiseReduction = "medium";
        // options.mpegNoiseReduction = "low";
        // options.blackLevel = "low";
        // options.gamma = "medium";

        this.setPictureProperty(options);

        _log("--> ControllBrightness: SET", options);
    };

    /**
     * Using API of TV LG for setting configurations
     * @param {Object} options
     */
    ControllBrightness.prototype.setPictureProperty = function setPictureProperty (options)
    {

        function successCb() {
            // console.log("Settings OK");
        }

        function failureCb(cbObject) {
            var errorCode = cbObject.errorCode;
            var errorText = cbObject.errorText;

            console.log ("Error Code [" + errorCode + "]: " + errorText);
        }

        var configuration = new Configuration();
        configuration.setPictureProperty(successCb, failureCb, options);
    }

    /**
     * Set configuration of brightness
     * @method configuration
     *
     * @param  {string} brightnessSettings       Format: [{init: 'hh:mm', bringhtness: 0-100}]
     */
    ControllBrightness.prototype.configuration = function __doneConfiguration (brightnessSettings)
    {
        var that = this;

        // get data
        this.getConfigurationStorage(function __doneGet(config) {

            if (!(that.keyConfiguration in config) || config[that.keyConfiguration] === null) {
                config[that.keyConfiguration] = {};
                config[that.keyConfiguration][that.dataPlayer.id] = {"itens": []};
            }

            config[that.keyConfiguration][that.dataPlayer.id].itens = [];

            async.each(brightnessSettings,
                function(brightness, nextBrightnessSetting){
                    // push item
                    config[that.keyConfiguration][that.dataPlayer.id].itens.push(brightness);
                    nextBrightnessSetting();
                },
                function doneBrightnessConfig(err){
                    window.brightness = config[that.keyConfiguration][that.dataPlayer.id];

                    // write data
                    that.setConfigurationStorage(config, function __doneApplyBrightness() {
                        // set configuration
                        that.init();
                    });
                }
            );
        });
    };

    /**
     * Clear configuration of local storage
     * @method clearConfiguration
     */
    ControllBrightness.prototype.clearConfiguration = function __doneClearConfiguration (callback)
    {
        // overwrite
        var ab = {};
        ab[this.keyConfiguration] = null;
        this.setConfigurationStorage(ab, callback);

        // clear timeouts
        this.clearTimeouts();
    };

    /**
     * Clear timeouts
     * @method clearTimeouts
     */
    ControllBrightness.prototype.clearTimeouts = function __doneClearTimeouts ()
    {
        // cancel timeouts
        if (!this.setIntervalConfig || !('length' in this.setIntervalConfig)) {
            return;
        }

        for(var i in this.setIntervalConfig) {
            clearTimeout(this.setIntervalConfig[i]);
        }
    };


    /**
     * Get data in local storage
     * @method getConfigurationStorage
     *
     * @param  {Function} callback
     */
    ControllBrightness.prototype.getConfigurationStorage = function __doneGetConfigurationStorage (callback)
    {
        player4uc.storage.get([this.keyConfiguration], callback);
    };

    /**
     * Write data in local storage
     * @method setConfigurationStorage
     *
     * @param {Object} object
     */
    ControllBrightness.prototype.setConfigurationStorage = function __doneSetConfigurationStorage (object, callback)
    {
        callback = callback || function __doneSetConfiStorage () {};

        player4uc.storage.set(object, callback);
    };

    _global.ControllBrightness = ControllBrightness;

}) (window);
