(function(_global) {

    /**
     * Classe para manipular arquivos em disco no chrome
     *
     * @class FileSystem
     * @constructor
     * @param {String} projPath Caminho do projeto
     */
    function FileSystem() {

    }

    FileSystem.prototype.init = function init(callback) {
        var that = this;

        if(that.DIRECTORY_ENTRY)
            return that.DIRECTORY_ENTRY;

        function openDirectory(directoryEntry) {

            if ( !directoryEntry )
                return;

            that.DIRECTORY_ENTRY = directoryEntry;
            callback(null, directoryEntry);

            // Armazena entrada do diretário para ser recuparado posteriormente.
            var retainEntry = chrome.fileSystem.retainEntry(directoryEntry);
            chrome.storage.local.set({ '__filesystem.retainEntry': retainEntry });


            // chrome.fileSystem.getDisplayPath(directoryEntry, function(displayPath) {
            //     console.log(displayPath, directoryEntry)
            // });
        }

        function getRetainEntry(retainEntry){

            if(!retainEntry || Object.keys(retainEntry).length == 0) {
                return chrome.fileSystem.chooseEntry({type: 'openDirectory'}, openDirectory);
            }

            retainEntry = retainEntry['__filesystem.retainEntry'];

            chrome.fileSystem.restoreEntry(retainEntry, function(directoryEntry) {
                if (!directoryEntry) {
                    return chrome.fileSystem.chooseEntry({type: 'openDirectory'}, openDirectory);
                }

                that.DIRECTORY_ENTRY = directoryEntry;
                callback(null, directoryEntry);
            });
        }

        chrome.storage.local.get(['__filesystem.retainEntry'], getRetainEntry);
    }

    /**
     * Creates a directory.
     *
     * @method mkdir
     * @param {string} path
     * @param {function} callback
     */
    FileSystem.prototype.mkdir = function (path, options, callback) {
        if(!this.DIRECTORY_ENTRY)
           return console.warn('Filesystem API not initialized');

        if(!path)
           return false;

        var folders = path.split('/');
        var dirEntry = this.DIRECTORY_ENTRY;

        _mkdir(dirEntry, folders);

        function _mkdir(dirEntry, folders) {
            if(folders.length == 0) {
                return callback(null);
            }

            dirEntry.getDirectory(folders[0], {
                create: true
            }, function(dirEntry) {
                folders.shift();
                _mkdir(dirEntry, folders);
            }, function(error){
                callback(error);
            });
        }
    }

    /**
     * Remove a directory.
     *
     * @method rmdir
     * @param {string} path
     * @param {object} options
     * @param {function} callback
     */
    FileSystem.prototype.rmdir = function (path, options, callback) {
        if(!this.DIRECTORY_ENTRY)
           return console.warn('Filesystem API not initialized');

        if(!path)
           return false;

        var dirEntry = this.DIRECTORY_ENTRY;
        var options = options || {};

        if(options.recursive === undefined)
            options.recursive = false;

        dirEntry.getDirectory(path, {}, function(dirEntry) {
            if(options.recursive){
                dirEntry.removeRecursively(function(){

                    if(callback) callback(null);
                }, errorHandler);

                return;
            }

            dirEntry.remove(function(){

                //call callback function if specified
                if(callback) callback();
            }, errorHandler);

        }, errorHandler);

        function errorHandler(error) {
            if(callback) callback(error);
        }
    }

    /**
     * Remove a file.
     *
     * @method rm
     * @param {string} filePath
     * @param {object} options
     * @param {function} callback
     */
    FileSystem.prototype.rm = function (filePath, options, callback) {
       if(!this.DIRECTORY_ENTRY)
           return console.warn('Filesystem API not initialized');

        if(!filePath)
           return false;

        var dirEntry = this.DIRECTORY_ENTRY;

        dirEntry.getFile(filePath, {create: false}, function(fileEntry) {

            fileEntry.remove(function() {

                if(callback) callback(error);

            }, errorHandler);

        }, errorHandler);

        function errorHandler(error) {
            if(callback) callback(error);
        }
    }

    /**
     * Read a file like text.
     *
     * @method readFile
     * @param {string} filePath
     * @param {function} callback
     */
    FileSystem.prototype.readFile = function (filePath, callback) {
        if(!this.DIRECTORY_ENTRY)
           return console.warn('Filesystem API not initialized');

        if(!filePath)
           return false;

        var dirEntry = this.DIRECTORY_ENTRY;

        dirEntry.getFile(filePath, {}, function(fileEntry) {

            fileEntry.file(function(file) {
                var reader = new FileReader();

                reader.onloadend = function(e) {
                    callback(null, this.result);
                };

                reader.onerror = function(e){
                    callback(e, null);
                };

                reader.readAsText(file);

            }, errorHandler);

        }, errorHandler);

        function errorHandler(error) {
            if(callback) callback(error);
        }
    }

    /**
     * Write a file.
     *
     * @method writeFile
     * @param {string} filePath
     * @param {string} data
     * @param {string} fileType
     * @param {object} options
     * @param {function} callback
     */
    FileSystem.prototype.writeFile = function(filePath, data, fileType, options, callback){

        if(typeof(fileType) != 'string')
            fileType = 'text/plain';

        if(typeof(fileType) == 'function')
            callback = fileType;

        if(typeof(options) != 'object')
            options = {};

        if(typeof(options) == 'function') {
            callback = options;
            options = {};
        }

        options.create = true;

        if(!this.DIRECTORY_ENTRY)
           return console.warn('Filesystem API not initialized');

        if(!filePath)
           return false;

        var dirEntry = this.DIRECTORY_ENTRY;

        dirEntry.getFile(filePath, options, function(fileEntry) {

            var blob = new Blob([data], {type: fileType});

            // Create a FileWriter object for our FileEntry (log.txt).
            fileEntry.createWriter(function(fileWriter) {

                fileWriter.onwriteend = function(e) {
                    if (fileWriter.length === 0) {
                        fileWriter.write(blob);
                    }

                    if(callback) callback(fileEntry);
                };

                fileWriter.onerror = errorHandler;

                // Clear content data file
                fileWriter.truncate(0);


            }, errorHandler);

        }, errorHandler);

        function errorHandler(error) {
            if(callback) callback(error);
        }
    }

   /**
     * Rename file.
     *
     * @method renameFile
     * @param {string} filePath
     * @param {string} newName
     * @param {function} callback
     */
    FileSystem.prototype.renameFile = function(filePath, newName, callback) {
        if(!this.DIRECTORY_ENTRY)
           return console.warn('Filesystem API not initialized');

        if(!filePath)
           return false;

        var dirEntry = this.DIRECTORY_ENTRY;

        var pathArray = filePath.split('/');
        var pLength = pathArray.length;
        var pathToParent = "";

        for(var i = 0; i<=pLength-2; i++) {
            pathToParent = pathToParent+pathArray[i]+"/";
        }

        dirEntry.getDirectory(pathToParent,{},function(parentDir){
            pathToParent = parentDir;
        },errorHandler);

        dirEntry.getFile(path, {}, function(fileEntry){
            fileEntry.moveTo(pathToParent, newName,function(){
                console.log('File renamed');

                //call callback function if specified
                if(callback) callback(fileEntry);
            }, errorHandler);
        }, errorHandler);

        function errorHandler(error) {
            if(callback) callback(error);
        }
    }

    /**
     * List all files that exists in the specified path.
     *
     * @param  {string}   path     path to list
     * @param  {Function} callback Callback function
     * @return {array}             Outputs an array of objects
     */
    FileSystem.prototype.ls = function ls(path, callback) {

        if(!this.DIRECTORY_ENTRY)
           return console.warn('Filesystem API not initialized');

        if(!path)
           return false;

        var dirEntry = this.DIRECTORY_ENTRY;
        var dirReader;
        var arr = [];
        var pathArray = path.split('/');
        var pLength = pathArray.length;
        var pathToParent= "";

        for(var i = 0; i<=pLength-1; i++){
            pathToParent = pathToParent+pathArray[i]+"/";
        }

        dirEntry.getDirectory(pathToParent,{},function(parentDir){
            pathToParent = parentDir;
            dirReader = (pathToParent) ? pathToParent.createReader() : fs.root.createReader();
            dirReader.readEntries(function(entries) {

                for (var i = 0, entry; entry = entries[i]; ++i) {
                    arr.push({
                        name: entry.name,
                        fileEntry: entry.filesystem
                    });
                }

                if(callback) callback(null, arr);
            }, errorHandler);
        },errorHandler);

        function errorHandler(error) {
            if(callback) callback(error);
        }
    }

    _global.FileSystem = FileSystem;
})(this);


// var fs = new FileSystem();
// fs.init(function(){
//      fs.ls('/', function(){});
// });
