/* globals Storage, _ */
(function(_global) {
    'use strict';

    /**
     * Classe to manipulate filesystem
     *
     * @class Filesystem
     * @constructor
     * @param {String} projPath Caminho do projeto
     */
    function Filesystem(options, callback) {
        this.init(options, callback);
    }

    Filesystem.prototype.init = function init(options, callback) {

        if(!this.STORAGE) {
            this.STORAGE = new Storage();
        }

        if(callback)
            callback(null, {success: true});
    };

    /**
     * Creates a directory.
     *
     * @method mkdir
     * @param {string} path
     * @param {object} options || callback
     * @param {function} callback
     */
    Filesystem.prototype.mkdir = function (path, options, callback) {
        if(!path)
           return callback({message: 'Invalid dir path'}, null);

        if(typeof options == 'function')
            callback = options;

        var onSuccess = function (){
            if(typeof callback == 'function')
                callback(null, {success: true});
        };

        var onError = function (error){
            if(typeof callback == 'function')
                callback(getError(error), null);
        };

        this.STORAGE.mkdir( onSuccess , onError , { path: normalizePath(path) } );
    };

    /**
     * Remove a directory.
     *
     * @method rmdir
     * @param {string} path
     * @param {object} options || callback
     * @param {function} callback
     */
    Filesystem.prototype.rmdir = function (path, options, callback) {
        if(!path)
            return callback({message: 'Invalid file path'}, null);

        if(typeof options == 'function')
            callback = options;


        var onSuccess = function (){
            if(typeof callback == 'function')
                callback(null, {success: true});
        };

        var onError = function (error){
            if(typeof callback == 'function')
                callback(getError(error), null);
        };

        options = (typeof options == 'object') ? options : {};
        options.file = normalizePath(path);
        options.recursive = ('recursive' in options) ? options.recursive : false;

        this.STORAGE.removeFile( onSuccess , onError , options );
    };

    /**
     * Remove a file.
     *
     * @method rm
     * @param {string} filePath
     * @param {object} options
     * @param {function} callback
     */
    Filesystem.prototype.rm = function (filePath, options, callback) {
        return this.rmdir(filePath, options, callback);
    };

    /**
     * Read a file like text.
     *
     * @method readFile
     * @param {string} filePath
     * @param {function} callback
     */
    Filesystem.prototype.readFile = function (filePath, noDecode, callback, options, statusFile) {
        if(!filePath)
            return callback({message: 'Invalid file filePath'}, null);

        if (typeof noDecode === 'function') {
            callback = noDecode;
        }

        // TODO: Explicando a GAMP
        // Como eu não sei o tamanho do arquivo, e o buffer maxímo da TV é 10k
        // vou realizar um loop de 3x pegando até 30k
        var that = this;
        var dataFile = '';
        var limitBuffer = 10240;
        var maxCount;

        // find status file
        if (!statusFile) {
            return this.statusFile(filePath, function(error, statusFile) {
                // caso error
                if (error) {
                    return callback(getError({
                        'errorCode': 0,
                        'errorText': 'File not found',
                    }), null);
                }

                // lenght file
                maxCount = statusFile.size;

                // read file
                read (filePath, noDecode, callback, options, statusFile);
            });
        }

        // find status file
        read (filePath, noDecode, callback, options, statusFile);

        function read (filePath, noDecode, callback, options, statusFile) {
            options = options || {};
            maxCount -= limitBuffer;

            var onSuccess = function (data){
                options.position += limitBuffer;

                dataFile += data.data;

                // caso todo o arquivo ainda não tenha sido lido
                if (maxCount > 0) {
                    return read (filePath, noDecode, callback, options, statusFile);
                }

                dataFile = (typeof noDecode === 'function'? decodeURI(dataFile) : dataFile);
                callback(null, dataFile);
                dataFile = null;
            };

            var onError = function(error) {
                if(typeof callback == 'function')
                    callback(getError(error), null);
            };

            options = extendsObject({
                path : normalizePath(filePath),
                position : 0,
                length : 10240,
                encoding: 'utf8'
            }, options);

            that.STORAGE.readFile( onSuccess , onError , options );
        }
    };

    /**
     * File exists
     *
     * @method exists
     * @param  {string}   filePath
     * @param  {Function} callback
     */
    Filesystem.prototype.exists = function (filePath, callback) {
        if(!filePath)
            return callback({message: 'Invalid file filePath'}, null);

        var onSuccess = function (data){
            callback = (_.isFunction(callback)? callback : function () {});

            // callback
            callback(data.exists);
        };

        var onError = function(error) {
            if(typeof callback == 'function')
                callback(getError(error), null);
        };

        var options = {
            path : normalizePath(filePath),
        };

        this.STORAGE.exists( onSuccess , onError , options );
    };

    /**
     * Write a file.
     *
     * @method writeFile
     * @param {string} path
     * @param {string} data
     * @param {string} fileType
     * @param {object} options
     * @param {function} callback
     */
    Filesystem.prototype.writeFile = function(filePath, data, fileType, options, callback){

        var that = this;

        if(!filePath)
            return callback({message: 'Invalid file filePath'}, null);

        if(typeof fileType == 'function')
            callback = fileType;

        if(typeof options == 'function')
            callback = options;

        var onSuccess = function (data){
            callback = callback || function() {};


            // caso a quantidade de caracters seja maior que 10k
            // é necessário que seja gravado no arquivo mais de uma vez
            var limitBuffer = 10240;
            options.position += limitBuffer;
            options.data = options.data.substr(limitBuffer);
            options.noEncode = true;
            options.mode = 'append';

            // caso não exista mais nada a ser gravado
            if (!options.data.length) {
            // if (options.data.length <= limitBuffer) {
                return callback(null, data);
            }

            that.writeFile(filePath, options.data, fileType, options, callback);
        };

        var onError = function(error) {
            if(typeof callback == 'function')
                callback(getError(error), null);
        };

        data = ((typeof options === 'object' && options.noEncode)? data : encodeURI(data));

        options = extendsObject({
            data: data,
            path: normalizePath(filePath),
            position : 0,
            mode :'truncate',
            // offset:0,
            // length : data.length,
            encoding: 'utf8'
        }, options);

        this.STORAGE.writeFile( onSuccess , onError , options );
    };

   /**
     * Rename file.
     *
     * @method renameFile
     * @param {string} filePath
     * @param {string} newName
     * @param {function} callback
     */
    Filesystem.prototype.renameFile = function(filePath, newFilePath, callback) {

        if(!filePath)
           return callback({message: 'Invalid dir filePath'}, null);

        var onSuccess = function (data){
            if(typeof callback == 'function')
                callback(null, data);
        };

        var onError = function(error){
            if(typeof callback == 'function')
                callback(getError(error), null);
        };

        var options = {
            oldPath: normalizePath(filePath),
            newPath : normalizePath(newFilePath)
        };

        this.STORAGE.moveFile( onSuccess , onError , options );
    };

    /**
     * List all files that exists in the specified path.
     *
     * @param  {string}   path     path to list
     * @param  {Function} callback Callback function
     * @return {array}             Outputs an array of objects
     */
    Filesystem.prototype.ls = function ls(path, callback) {
        var list = [];

        if(!path)
           return callback({message: 'Invalid file path'}, null);

        var onSuccess = function (data){

            for (var i = 0; i < data.files.length; i++) {
                var file = data.files[i];
                list.push({
                    name: file.name,
                    size: file.size,
                    isDirectory: (file.type == 'folder') ? true : false,
                    isFile: (file.type == 'file') ? true : false
                });
            }

            if(typeof callback == 'function')
                callback(null, list);
        };

        var onError = function (error){
            if(typeof callback == 'function')
                callback(getError(error), null);
        };

        this.STORAGE.listFiles( onSuccess , onError , { path: normalizePath(path) } );
    };

    /**
     * Status of files
     *
     * @param  {string}   path
     * @param  {Function} callback
     */
    Filesystem.prototype.statusFile = function statusFile(path, callback) {
        var successCb = function (cbObject){
            if(typeof callback == 'function')
                callback(null, cbObject);
        };

        var failureCb = function(cbObject){
            if(typeof callback == 'function')
                callback(getError(cbObject), null);
        };

        var options = {
            path: normalizePath(path)
        };

        this.STORAGE.statFile(successCb, failureCb, options);
    };


    /**
     * Convert path string to correct format
     * @param  {String} pathString [description]
     * @example            normalizePath('file://internal')         file://internal
     * @example            normalizePath('file://internal/anyone')  file://internal/anyone
     * @example            normalizePath('/anyone')                 file://internal/anyone
     * @example            normalizePath('anyone')                  file://internal/anyone
     * @return {String}
     */
    function normalizePath (pathString) {

        if(!pathString.match('file://'))
            pathString = (pathString[0] == '/')    ?    pathString    :    '/' + pathString;

        pathString = (pathString[pathString.length -1] == '/')    ?    pathString.substring(0, pathString.length-1)    :    pathString;

        if(!pathString.match('file://internal'))
            pathString = pathString.match('file://internal') ? pathString : 'file://internal' + pathString;

        return pathString;
    }

    function getError(error) {
        if(typeof error != 'object')
            return error;

        if(!('errorCode' in error) && !('errorText' in error))
            return error;

        return {
            code: error.errorCode,
            message: error.errorText
        };
    }

    /**
     * Method for extends object
     * @param  {Object} objectExt
     * @param  {Object} object
     * @return {Object}
     */
    function extendsObject (objectExt, object) {
        if (typeof objectExt !== 'object' || typeof object !== 'object') {
            return objectExt;
        }

        for (var i in object) {
            objectExt[i] = object[i];
        }

        return objectExt;
    }

    _global.Filesystem = Filesystem;
})(this);


/**
 *
 *  var fs = new Filesystem();
 *      fs.init(function(){
 *          fs.ls('/', function(){});
 *       });
 *
 **/
